#include <xgpio_l.h>
#include <xparameters.h>
#include <xuartns550_l.h>
#include "peripheral.h"

#define LED_BASEADDR    XPAR_LEDS_4BIT_BASEADDR
#define SWITCH_BASEADDR XPAR_DIPSWS_4BIT_BASEADDR

void init_uart()
{
    XUartNs550_SetBaud(UART_BASEADDR, UART_CLOCK, UART_BAUDRATE);
    XUartNs550_mSetLineControlReg(UART_BASEADDR, XUN_LCR_8_DATA_BITS);
}

void initLedDip() {


    XGpio_mSetDataDirection(LED_BASEADDR, 1, 0x0);

    XGpio_mSetDataDirection(SWITCH_BASEADDR, 1, 0xFF);
    
    turnLED(-1, 0);
}

//Helper method, turn the specified LED on or off
void turnLED(int ledNumber, int on){
    Xuint32 data = 0xffffffff;
   
    if(ledNumber == -1){
        if(on)
            data = 0;
        else
            data = 0xFFFFFFFF;        
    }
    else{
            switch (ledNumber){
            case 0: data =  0xFFFFFFF7; break;
            case 1: data =  0xFFFFFFFB; break;
            case 2: data =  0xFFFFFFFD; break;
            case 3: data =  0xFFFFFFFE; break;
            default: data = 0xffffffff;
		}		 
    }
 
    XGpio_mSetDataReg(LED_BASEADDR, 1, data);
}

Xuint8 getDIP(int dipNumber)
{
    Xuint32 data;
    data = XGpio_mGetDataReg(SWITCH_BASEADDR, 1);
    data = (data>>(6-dipNumber)) & 0x1;
    return (Xuint8)data;
}


void testLedDip()
{
    int i, delay;
    Xuint8 dip;
    
    turnLED(-1, 1);
    for(delay=0; delay<50000; delay++);
    turnLED(-1, 0);
    
    while(1){{
        char s = 'a';

        xil_printf("Input the LED to turn on (0-7):");
        s = XUartNs550_RecvByte(UART_BASEADDR);
        xil_printf("%d\r\n",s);

        turnLED(-1, 0);
        s = s - '0';
        turnLED(s, 1);

        for(i=0; i<=6; i++){
            dip = getDIP(i);
            xil_printf("DIP %d = %d  ", i, dip);
        }
        xil_printf("\r\n");
    }
    
    }
}
