#define TESTAPP_GEN

/* $Id: xemac_polled_example.c,v 1.2 2004/12/22 21:44:53 moleres Exp $ */
/*****************************************************************************
*
*       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*       FOR A PARTICULAR PURPOSE.
*
*       (c) Copyright 2002-2004 Xilinx Inc.
*       All rights reserved.
*
*****************************************************************************/
/****************************************************************************/
/**
*
* @file xemac_polled_example.c
*
* Contains an example of how to use the XEmac driver directly. The example here
* shows using the driver/device in polled mode.
*
* @note
*
* None.
*
******************************************************************************/

/***************************** Include Files *********************************/

#include "xemac.h"
#include "xparameters.h"
#include "xstatus.h"


/************************** Constant Definitions *****************************/

#define XEM_MAX_FRAME_SIZE_IN_WORDS ((XEM_MAX_FRAME_SIZE / sizeof(Xuint32)) + 1)

/**************************** Type Definitions *******************************/


/***************** Macros (Inline Functions) Definitions *********************/


/************************** Function Prototypes ******************************/

XStatus XEmacPolledExample(Xuint16 DeviceId);
static XStatus LoopbackFrame(XEmac *InstancePtr, Xuint32 PayloadSize);
static XStatus SendFrame(XEmac *InstancePtr, Xuint32 PayloadSize,
                         Xuint8 *DestAddress);
static XStatus RecvFrame(XEmac *InstancePtr, Xuint32 PayloadSize);

/************************** Variable Definitions *****************************/

/*
 * The loopback test will use the LocalAddress both as source and destination
 * MAC address.
 */
static Xuint8 LocalAddress[XEM_MAC_ADDR_SIZE] =
{
    0x06, 0x05, 0x04, 0x03, 0x02, 0x01
};

/* Frame buffers */
static Xuint32 TxFrame[XEM_MAX_FRAME_SIZE_IN_WORDS];    /* word aligned */
static Xuint32 RxFrame[XEM_MAX_FRAME_SIZE_IN_WORDS];    /* word aligned */

/* Driver instance */
XEmac Emac;

/******************************************************************************
*
* Main function to call the example.  This function is not included if the
* example is generated from the TestAppGen test tool.
*
******************************************************************************/
#ifndef TESTAPP_GEN
int main(void)
{
    (void)XEmacPolledExample(XPAR_ETHERNET_MAC_DEVICE_ID);

    return 0;
}
#endif

/*****************************************************************************/
/**
*
* The entry point for showing the XEmac driver in polled mode. The example
* configures the device for internal loopback mode, then sends an Ethernet
* frame and receives the same Ethernet frame.
*
* @param DeviceId is the XPAR_<emac_instance>_DEVICE_ID value from
*        xparameters.h
*
* @return
*
* 0 if successful, otherwise the number of errors that occurred.
*
* @note
*
* None.
*
******************************************************************************/
XStatus XEmacPolledExample(Xuint16 DeviceId)
{
    XEmac *InstancePtr = &Emac;
    XStatus Result;
    Xuint32 Options;                            /* device options */

    /*
     * Initialize the XEmac component. The device ID chosen should be
     * configured without DMA (see the xemac_g.c file)
     */
    Result = XEmac_Initialize(InstancePtr, DeviceId);
    if (Result != XST_SUCCESS)
    {
        return Result;
    }

    /*
     * Run self-test on the device, which verifies basic sanity of the
     * device and the driver.
     */
    Result = XEmac_SelfTest(InstancePtr);
    if (Result != XST_SUCCESS)
    {
        return Result;
    }

    /*
     * First configure the device into loopback mode.  We also configure it
     * for unicast and broadcast addressing.  Also, set the MAC address.
     */
    Options = XEmac_GetOptions(InstancePtr);
    Options |= (XEM_POLLED_OPTION | XEM_LOOPBACK_OPTION);

    Result = XEmac_SetOptions(InstancePtr, Options);
    if (Result != XST_SUCCESS)
    {
        return Result;
    }

    Result = XEmac_SetMacAddress(InstancePtr, LocalAddress);
    if (Result != XST_SUCCESS)
    {
        return Result;
    }

    /*
     * Start the device, which enables the transmitter and receiver
     */
    Result = XEmac_Start(InstancePtr);
    if (Result != XST_SUCCESS)
    {
        return Result;
    }

    /*
     * We clear the driver statistics so we can look at them later
     */
    XEmac_ClearStats(InstancePtr);

    /*
     * Loopback a 200-byte frame (218 with the Ethernet header/trailer).
     */
    Result = LoopbackFrame(InstancePtr, 200);
    if (Result != XST_SUCCESS)
    {
        return Result;
    }

    /*
     * Stop the device, which disables the transmitter and receiver
     */
    Result = XEmac_Stop(InstancePtr);
    if (Result != XST_SUCCESS)
    {
        return Result;
    }

    return Result;
}


/******************************************************************************
*
* Loopback a frame of given size. This function assumes polled mode and
* loopback mode, sends the frame, then receives the frame and verifies its
* contents.
*
* @param InstancePtr is a pointer to the driver instance
* @param PayloadSize is the size of the frame to create. The size only reflects
*        the payload size, it does not include the Ethernet header size (14
*        bytes) nor the Ethernet CRC size (4 bytes).
*
* @return
*
* XST_SUCCESS if successful, a driver-specific return code if not
*
* @note
*
* None.
*
******************************************************************************/
static XStatus LoopbackFrame(XEmac *InstancePtr, Xuint32 PayloadSize)
{
    XStatus Result;

    Result = SendFrame(InstancePtr, PayloadSize, LocalAddress);
    if (Result == XST_SUCCESS)
    {
        Result = RecvFrame(InstancePtr, PayloadSize);
    }

    return Result;
}

/******************************************************************************
*
* Send a frame of given size. This function assumes polled mode and sends the
* frame. The caller must be aware of FIFO depths when using this function.
*
* @param InstancePtr is a pointer to the driver instance
* @param PayloadSize is the size of the frame to create. The size only reflects
*        the payload size, it does not include the Ethernet header size (14
*        bytes) nor the Ethernet CRC size (4 bytes).
* @param DestAddress is the destination MAC address for the frame
*
* @return
*
* XST_SUCCESS if successful, a driver-specific return code if not
*
* @note
*
* None.
*
******************************************************************************/
static XStatus SendFrame(XEmac *InstancePtr, Xuint32 PayloadSize,
                         Xuint8 *DestAddress)
{
    Xuint8 *FramePtr;
    Xuint8 *AddrPtr = DestAddress;
    int Index;

    /*
     * Assemble the frame with a destination address and a bogus source address
     * (the MAC overwrites it by default)
     */
    FramePtr = (Xuint8 *)TxFrame;

    *FramePtr++ = *AddrPtr++;
    *FramePtr++ = *AddrPtr++;
    *FramePtr++ = *AddrPtr++;
    *FramePtr++ = *AddrPtr++;
    *FramePtr++ = *AddrPtr++;
    *FramePtr++ = *AddrPtr++;

    FramePtr += XEM_MAC_ADDR_SIZE;  /* get past source address (bogus) */

    /* Set up the type/length field - be sure its in network order */
    *((Xuint16 *)FramePtr) = PayloadSize;
    FramePtr++;
    FramePtr++;

    /*
     * Now fill in the data field with known values so we can verify them
     * on receive.
     */
    for (Index = 0; Index < PayloadSize; Index++)
    {
        *FramePtr++ = (Xuint8)Index;
    }

    /*
     * Now send the frame, then receive it and verify its contents
     */
    return XEmac_PollSend(InstancePtr, (Xuint8 *)TxFrame,
                          PayloadSize + XEM_HDR_SIZE);
}


/******************************************************************************
*
* Receive a frame of given size. This function assumes polled mode, receives
* the frame and verifies its contents.
*
* @param InstancePtr is a pointer to the driver instance
* @param PayloadSize is the size of the frame to create. The size only reflects
*        the payload size, it does not include the Ethernet header size (14
*        bytes) nor the Ethernet CRC size (4 bytes).
*
* @return
*
* XST_SUCCESS if successful, a driver-specific return code if not
*
* @note
*
* None.
*
******************************************************************************/
static XStatus RecvFrame(XEmac *InstancePtr, Xuint32 PayloadSize)
{
    Xuint8 *FramePtr;
    XStatus Result;
    Xuint32 RecvFrameLength;

    /* this assumes MAC does not strip padding or crc */
    RecvFrameLength = XEM_MAX_FRAME_SIZE;

    Result = XEmac_PollRecv(InstancePtr, (Xuint8 *)RxFrame, &RecvFrameLength);
    if (Result == XST_SUCCESS)
    {
        int Index;

        /* Verify length, which should be the payload plus overhead */
        if (RecvFrameLength != (PayloadSize + XEM_HDR_SIZE + XEM_TRL_SIZE))
        {
            return XST_LOOPBACK_ERROR;
        }

        /* Verify the frame contents */
        FramePtr = (Xuint8 *)RxFrame;
        FramePtr += XEM_HDR_SIZE;    /* get past the header */
        for (Index = 0; Index < PayloadSize; Index++)
        {
            if (*FramePtr++ != (Xuint8)Index)
            {
                return XST_LOOPBACK_ERROR;
            }
        }
     }

    return Result;
}

